; Hardware/driver routines for BMOW real-time clock

; periodic interrupt, 30.5us to 500ms interval
; alarm interrupt, 1s to 1 month interval
; power-fail interrupt

; --- memory-mapped RTC addresses
; T = tens, N = ones

; seconds: 0 TTT NNNN
RTC_ADR_SECONDS = $003F20

; seconds alarm
; Any alarm register can be maksed/disabled by setting both its D7 and D6 to 1.
; Registers must be masked in order from day to hour, minute, and second. The
; alarm is considered once every second, and is triggered if all the non-masked
; alarm registers match the current time values. 
; 0 TTT NNNN
RTC_ADR_SECONDS_ALARM = $003F21

; minutes: 0 TTT NNNN
RTC_ADR_MINUTES = $003F22

; minutes alarm: 0 TTT NNNN
RTC_ADR_MINUTES_ALARM = $003F23

; hours: PM/AM 0 TT NNNN
; 0 = AM, 1 = PM
RTC_ADR_HOURS = $003F24

; hours alarm: PM/AM 0 TT NNNN
RTC_ADR_HOURS_ALARM = $003F25

; day: 0 0 TT NNNN
; 0 = AM, 1 = PM
RTC_ADR_DAY = $003F26

; day alarm: 0 0 TT NNNN
RTC_ADR_DAY_ALARM = $003F27

; day of week: 00000 NNN
; 1 = Sunday, 7 = Saturday
RTC_ADR_DAY_OF_WEEK = $003F28

; month: 000 T NNNN
RTC_ADR_MONTH = $003F29

; year: TTTT NNNN
RTC_ADR_YEAR = $003F2A

; programmable rates: * WD2 WD1 WD0 RS3 RS2 RS1 RS0
; WD = watch dog rate
; RS : periodic interrupt rate
; 0 none
; 1 30.5175us
; 2 61.035us
; 3 122.07us
; 4 244.141us
; 5 488.281us
; 6 976.5625us
; 7 1.95315ms
; 8 3.90625ms
; 9 7.8125ms
; 10 15.625ms
; 11 31.25ms
; 12 62.5ms
; 13 125ms
; 14 250ms
; 15 500ms
RTC_ADR_RATES = $003F2B

; interrupt enable: * * * * AIE PIE PWRIE ABE
; ABE = alarm interrupt enable during battery backup mode
RTC_ADR_INTERRUPT_ENABLE = $003F2C

; flags: * * * * AF PF PWRF BVF
; BVF = battery valid flag
; reading the flags register clears all flags, and makes IRQ hi-Z
RTC_ADR_FLAGS = $003F2D

; control: * * * * UTI /STOP 24/12 DSE
; set UTI bit high before reading or writing the time
; DSE = daylight savings enable
RTC_ADR_CONTROL = $003F2E

; --- Procedure IDs for RTC procedure handler	
HANDLER_RTC_GETTIME = 0
	
; --- RTC print time
rtc_print_time:
	; day of week
	phb
	bank #$00
	lda RTC_ADR_DAY_OF_WEEK
	clc
	adc $3F28
	adc $3F28 ; A = day * 3
	plb
	tax
	dex
	dex
	dex ; make day 0 based
	lda rtc_daystrings, X
	phx
	jsr console_write_char
	plx
	inx
	lda rtc_daystrings, X
	phx
	jsr console_write_char
	plx
	inx
	lda rtc_daystrings, X
	jsr console_write_char
	
	lda #$20
	jsr console_write_char
	
	; month
	lda RTC_ADR_MONTH
	jsr console_write_hex
	lda #'/'
	jsr console_write_char
	
	; day
	lda RTC_ADR_DAY
	jsr console_write_hex
	lda #'/'
	jsr console_write_char
	
	; year
	lda #$A0
	phb
	bank #$00
	cmp $3F2A
	plb
	bcc +
	lda #$19
	jsr console_write_hex
	lda RTC_ADR_YEAR
	bra ++
+	lda #$20
	jsr console_write_hex
	lda RTC_ADR_YEAR
	sec
	sbc #$A0
++	jsr console_write_hex
	
	lda #$20
	jsr console_write_char
	
	; hour
	lda RTC_ADR_HOURS
	and #$7F
	jsr console_write_hex
	lda #':'
	jsr console_write_char

	; minute
	lda RTC_ADR_MINUTES
	jsr console_write_hex
		
	lda #$20
	jsr console_write_char
	
	; AM/PM
	lda RTC_ADR_HOURS
	bmi +
	ldx #$00
	bra ++
+	ldx #$02
++	lda rtc_ampmstrings, X
	phx
	jsr console_write_char
	plx
	inx
	lda rtc_ampmstrings, X
	jsr console_write_char
		
	rts	

rtc_daystrings:	
!text "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"

rtc_ampmstrings:
!text "AM", "PM"